<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kcalc" version="20">
<MenuBar>
  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="mode_simple"/>
    <Action name="mode_science"/>
    <Action name="mode_statistics"/>
    <Action name="mode_numeral"/>
    <Separator/>
    <Action name="show_constants"/>
    <Action name="show_bitset"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure"/>
  </Menu>
</MenuBar>
</gui>
